@@ -31,16 +31,16 @@ def test_cutoff_warning():
 
     cutoff = CutOff()
 
-    with pytest.warns(UserWarning):
+    with pytest.raises(ValueError):
         cutoff.fit_transform(X_data)
 
 
 def test_cutoff_transformer():
     cutoff = CutOff()
 
-    X_data = [1, 2, 0.5, 0.7, 100, -1, -23, 0]
+    X_data = np.array([1, 2, 0.5, 0.7, 100, -1, -23, 0]).reshape(-1,1)
 
-    assert np.all(cutoff.fit_transform(X_data) == [1, 1, 0.5, 0.7, 1, 0, 0, 0])
+    assert np.all(cutoff.fit_transform(X_data) == np.array([1, 1, 0.5, 0.7, 1, 0, 0, 0]).reshape(-1,1))
 
 
 def test_cutoff_inside_a_pipeline(data):
