@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 import argparse
 import importlib.util
 import os
@@ -439,7 +438,3 @@ class Db(Command):
 
 def execute():
     main(Triage)
-
-
-if __name__ == "__main__":
-    main(Triage)
