@@ -1,4 +1,5 @@
-import logging
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
 
 from triage.component.architect.database_reflection import table_has_data
 from triage.database_reflection import table_row_count, table_exists
@@ -40,32 +41,21 @@ class EntityDateTableGenerator:
             as_of_dates (list of datetime.dates) Dates to include in the
                 state table
         """
-        logging.debug(
-            "Generating entity_date table %s using as_of_dates: %s",
-            self.entity_date_table_name,
-            as_of_dates,
-        )
+        logger.spam(f"Generating entity_date table {self.entity_date_table_name} using as_of_dates: {as_of_dates}")
         self._create_and_populate_entity_date_table(as_of_dates)
-        self.db_engine.execute(
-            "create index on {} (entity_id, as_of_date)".format(self.entity_date_table_name)
-        )
-        logging.info(
-            "Indices created on entity_id and as_of_date for entity_date table %s",
-            self.entity_date_table_name,
-        )
+        logger.spam(f"Table {self.entity_date_table_name} created and populated")
+
         if not table_has_data(self.entity_date_table_name, self.db_engine):
             raise ValueError(self._empty_table_message(as_of_dates))
 
-        logging.info("Entity-date table generated at %s", self.entity_date_table_name)
-        logging.info("Generating stats on %s", self.entity_date_table_name)
-        logging.info(
-            "Row count of %s: %s",
-            self.entity_date_table_name,
-            table_row_count(self.entity_date_table_name, self.db_engine),
-        )
+        logger.debug(f"Entity-date table generated at {self.entity_date_table_name}")
+        logger.spam(f"Generating stats on {self.entity_date_table_name}")
+        logger.spam(f"Row count of {self.entity_date_table_name}: {table_row_count(self.entity_date_table_name, self.db_engine)}")
+
 
     def _maybe_create_entity_date_table(self):
         if self.replace or not table_exists(self.entity_date_table_name, self.db_engine):
+            logger.spam(f"Creating entity_date table {self.entity_date_table_name}")
             self.db_engine.execute(f"drop table if exists {self.entity_date_table_name}")
             self.db_engine.execute(
                 f"""create table {self.entity_date_table_name} (
@@ -75,12 +65,15 @@ class EntityDateTableGenerator:
                 )
                 """
             )
-            logging.info("Created entity_date table %s", self.entity_date_table_name)
+
+            logger.spam(f"Creating indices on entity_id and as_of_date for entity_date table {self.entity_date_table_name}")
+            self.db_engine.execute(
+                f"create index on {self.entity_date_table_name} (entity_id, as_of_date)"
+            )
         else:
-            logging.info(
-                "Not dropping and recreating entity_date %s table because "
-                "replace flag was set to False and table was found to exist",
-                self.entity_date_table_name,
+            logger.notice(
+                f"Not dropping and recreating entity_date {self.entity_date_table_name} table because "
+                f"replace flag was set to False and table was found to exist"
             )
 
     def _create_and_populate_entity_date_table(self, as_of_dates):
@@ -91,10 +84,10 @@ class EntityDateTableGenerator:
         as_of_dates (list of datetime.date): Dates to calculate entity states as of
         """
         self._maybe_create_entity_date_table()
-        logging.info("Inserting rows into entity_date table %s", self.entity_date_table_name)
+        logger.spam(f"Inserting rows into entity_date table {self.entity_date_table_name}")
         for as_of_date in as_of_dates:
             formatted_date = f"{as_of_date.isoformat()}"
-            logging.info("Looking for existing entity_date rows for as of date %s", as_of_date)
+            logger.spam(f"Looking for existing entity_date rows for as of date {as_of_date}")
             any_existing = list(self.db_engine.execute(
                 f"""select 1 from {self.entity_date_table_name}
                 where as_of_date = '{formatted_date}'
@@ -102,7 +95,7 @@ class EntityDateTableGenerator:
                 """
             ))
             if len(any_existing) == 1:
-                logging.info("Since >0 entity_date rows found for date %s, skipping", as_of_date)
+                logger.spam(f"Since >0 entity_date rows found for date {as_of_date}, skipping")
                 continue
             dated_query = self.query.format(as_of_date=formatted_date)
             full_query = f"""insert into {self.entity_date_table_name}
@@ -110,7 +103,7 @@ class EntityDateTableGenerator:
                 from ({dated_query}) q
                 group by 1, 2, 3
             """
-            logging.info("Running entity_date query for date: %s, %s", as_of_date, full_query)
+            logger.spam(f"Running entity_date query for date: {as_of_date}, {full_query}")
             self.db_engine.execute(full_query)
 
     def _empty_table_message(self, as_of_dates):
@@ -127,21 +120,21 @@ class EntityDateTableGenerator:
         )
 
     def clean_up(self):
-        self.db_engine.execute("drop table if exists {}".format(self.entity_date_table_name))
+        self.db_engine.execute(f"drop table if exists {self.entity_date_table_name}")
 
 
-class EntityDateTableGeneratorNoOp(EntityDateTableGenerator):
+class CohortTableGeneratorNoOp(EntityDateTableGenerator):
     def __init__(self):
         pass
 
     def generate_entity_date_table(self, as_of_dates):
-        logging.warning(
+        logger.warning(
             "No cohort configuration is available, so no cohort will be created"
         )
         return
 
     def clean_up(self):
-        logging.warning("No cohort table exists, so nothing to tear down")
+        logger.warning("No cohort configuration is available, so no cohort will be tear down")
         return
 
     @property
