@@ -1,19 +1,20 @@
-import logging
-import textwrap
-from triage.database_reflection import table_exists
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
 
+import textwrap
+from triage.database_reflection import table_row_count, table_exists
 
 DEFAULT_LABEL_NAME = "outcome"
 
 
 class LabelGeneratorNoOp:
     def generate_all_labels(self, labels_table, as_of_dates, label_timespans):
-        logging.warning(
+        logger.warning(
             "No label configuration is available, so no labels will be created"
         )
 
     def generate(self, start_date, label_timespan, labels_table):
-        logging.warning(
+        logger.warning(
             "No label configuration is available, so no labels will be created"
         )
 
@@ -32,76 +33,63 @@ class LabelGenerator:
 
     def _create_labels_table(self, labels_table_name):
         if self.replace or not table_exists(labels_table_name, self.db_engine):
-            self.db_engine.execute("drop table if exists {}".format(labels_table_name))
+            self.db_engine.execute(f"drop table if exists {labels_table_name}")
             self.db_engine.execute(
-                """
-                create table {} (
+                f"""
+                create table {labels_table_name} (
                 entity_id int,
                 as_of_date date,
                 label_timespan interval,
-                label_name varchar(30),
-                label_type varchar(30),
-                label int
-            )""".format(
-                    labels_table_name
-                )
+                label_name varchar,
+                label_type varchar,
+                label smallint
+                )"""
             )
         else:
-            logging.info("Not dropping and recreating table because "
-                         "replace flag was set to False and table was found to exist")
+            logger.notice(f"Not dropping and recreating {labels_table_name} table because "
+                          f"replace flag was set to False and table was found to exist")
 
     def generate_all_labels(self, labels_table, as_of_dates, label_timespans):
         self._create_labels_table(labels_table)
-        logging.info(
-            "Creating labels for %s as of dates and %s label timespans",
-            len(as_of_dates),
-            len(label_timespans),
-        )
+        logger.spam(f"Creating labels for {len(as_of_dates)} as of dates and {len(label_timespans)} label timespans")
         for as_of_date in as_of_dates:
             for label_timespan in label_timespans:
                 if not self.replace:
-                    logging.info(
-                        "Looking for existing labels for as of date %s and label timespan %s",
-                        as_of_date,
-                        label_timespan,
-                    )
+                    logger.spam(f"Looking for existing labels for as of date {as_of_date} and label timespan {label_timespan}")
                     any_existing_labels = list(self.db_engine.execute(
-                        """select 1 from {labels_table}
+                        f"""select 1 from {labels_table}
                         where as_of_date = '{as_of_date}'
                         and label_timespan = '{label_timespan}'::interval
-                        and label_name = '{label_name}'
+                        and label_name = '{self.label_name}'
                         limit 1
-                        """.format(
-                            labels_table=labels_table,
-                            as_of_date=as_of_date,
-                            label_timespan=label_timespan,
-                            label_name=self.label_name
-                        )
-                    ))
+                        """)
+                    )
                     if len(any_existing_labels) == 1:
-                        logging.info("Since nonzero existing labels found, skipping")
+                        logger.spam("Since nonzero existing labels found, skipping")
                         continue
 
-                logging.info(
-                    "Generating labels for as of date %s and label timespan %s",
-                    as_of_date,
-                    label_timespan,
+                logger.debug(
+                    f"Generating labels for as of date {as_of_date} and label timespan {label_timespan}",
                 )
                 self.generate(
                     start_date=as_of_date,
                     label_timespan=label_timespan,
                     labels_table=labels_table,
                 )
-        nrows = [
-            row[0]
-            for row in self.db_engine.execute(
-                "select count(*) from {}".format(labels_table)
-            )
-        ][0]
+
+        self.db_engine.execute(
+            f"create index on {labels_table} (entity_id, as_of_date)"
+        )
+        logger.spam("Added index to labels table")
+
+        nrows = table_row_count(labels_table, self.db_engine)
+
         if nrows == 0:
-            logging.warning("Done creating labels, but no rows in labels table!")
+            logger.warning(f"Done creating labels, but no rows in {labels_table} table!")
+            raise ValueError(f"{label_table} is empty!")
         else:
-            logging.info("Done creating labels table %s: rows: %s", labels_table, nrows)
+            logger.debug(f"Labels table generated at {labels_table}")
+            logger.spam(f"Row count of {labels_table}: nrows")
 
     def generate(self, start_date, label_timespan, labels_table):
         """Generate labels table using a query
@@ -123,28 +111,22 @@ class LabelGenerator:
         )
 
         full_insert_query = textwrap.dedent(
-            """
+            f"""
             insert into {labels_table}
             select
                 entities_and_outcomes.entity_id,
-                '{as_of_date}' as as_of_date,
+                '{start_date}' as as_of_date,
                 '{label_timespan}'::interval as label_timespan,
-                '{label_name}' as label_name,
+                '{self.label_name}' as label_name,
                 'binary' as label_type,
                 entities_and_outcomes.outcome as label
-            from ({user_query}) entities_and_outcomes
-        """
-        ).format(
-            user_query=query_with_db_variables,
-            labels_table=labels_table,
-            as_of_date=start_date,
-            label_timespan=label_timespan,
-            label_name=self.label_name,
+            from ({query_with_db_variables}) entities_and_outcomes
+            """
         )
 
-        logging.debug("Running label creation query")
-        logging.debug(full_insert_query)
+        logger.spam("Running label insertion query")
+        logger.spam(full_insert_query)
         self.db_engine.execute(full_insert_query)
 
     def clean_up(self, labels_table_name):
-        self.db_engine.execute("drop table if exists {}".format(labels_table_name))
+        self.db_engine.execute(f"drop table if exists {labels_table_name}")
