@@ -1,3 +1,6 @@
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
+
 import datetime
 import shutil
 import sys
@@ -10,13 +13,15 @@ import tempfile
 import sqlalchemy
 
 import pandas as pd
-import numpy
+import numpy as np
 from sqlalchemy.orm import sessionmaker
 
 from triage.component.results_schema import Model
 from triage.util.structs import FeatureNameList
 
 
+
+
 def str_in_sql(values):
     return ",".join(map(lambda x: "'{}'".format(x), values))
 
@@ -104,7 +109,7 @@ def create_entity_date_df(
         for date in ids_dates["as_of_date"]
     ]
     ids_dates = ids_dates[ids_dates["as_of_date"].isin(as_of_dates)]
-    print(ids_dates)
+    logger.spam(ids_dates)
 
     return ids_dates.reset_index(drop=True)
 
@@ -126,12 +131,12 @@ def TemporaryDirectory():
 
 
 def fake_labels(length):
-    return numpy.array([random.choice([True, False]) for i in range(0, length)])
+    return np.array([random.choice([True, False]) for i in range(0, length)])
 
 
 class MockTrainedModel:
     def predict_proba(self, dataset):
-        return numpy.random.rand(len(dataset), len(dataset))
+        return np.random.rand(len(dataset), len(dataset))
 
 
 def fake_trained_model(project_path, model_storage_engine, db_engine):
